#!/bin/bash

CONF_FILE="/etc/condor/config.d/test"
TEST_FILE="test.sub"
ERR=0

function get_number {
  condor_userprio -l | grep "^Name.*$1\.[xy]" | sed -e "s/^Name\([0-9]\+\) = \".*$1\.[xy].*@.*/\1/"
  return $?
}

function get_attr {
  condor_userprio -l | grep "^${2}${1} = " | awk '{print $3}'
}

function get_numbers_of_ungrouped_users {
  condor_userprio -l | grep "^Name.*@" | grep -v "group" | sed -e "s/^Name\([0-9]\+\) = \".*@.*/\1/"
  return $?
}

function test_resources {
  local ret=0
  for i in 1 2 11 12 21 22; do 
# condor_userprio
    local number=$(get_number "group_$i")
    if [ -n "${number}" ]; then
      local attr=$(get_attr ${number} "ResourcesUsed")
      if [ -z "${attr}" ]; then
        attr="0"
      fi
      compare ${attr} ${groups[$i]} 0
      if [ "$?" -gt "0" -a "${groups_auto[$i]}" == "FALSE" ] ; then
        echo "1##########################"
        echo "groups : ${groups[@]}"
        echo "wrong number of jobs from group $i: ${attr} <= ${groups[$i]}"
        condor_userprio -all
        condor_userprio -l
        echo "##########################"
        let "ret++"
      fi
    fi
# condor_status
    attr=$(condor_status -format "%s\n" "AccountingGroup" | grep "group_$i\.[xy]" | wc -l)
    compare ${attr} ${groups[$i]} 0
    if [ "$?" -gt "0" -a "${groups_auto[$i]}" == "FALSE" ] ; then
      echo "2##########################"
      echo "groups : ${groups[@]}"
      echo "wrong number of jobs from group $i: ${attr} <= ${groups[$i]}"
      condor_status -format "%s\n" "AccountingGroup" | grep "group_$i\.[xy]"
      echo "##########################"
      let "ret++"
    fi
  done

  if [ "$TYPE" == "dynamic" ]; then
    for grp in 1 2; do
# condor_userprio
      local usage=0
      for sub in 1 2; do
         local number=$(get_number "group_${grp}${sub}")
         if [ -n "${number}" ]; then
           local attr=$(get_attr ${number} "ResourcesUsed")
           usage=$(echo -e "scale=100\n${usage} + ${attr}" | bc)
         fi
      done
      local number=$(get_number "group_${grp}")
      if [ -n "${number}" ]; then
        local attr=$(get_attr ${number} "ResourcesUsed")
        usage=$(echo -e "scale=100\n${usage} + ${attr}" | bc)
        compare ${usage} ${groups[${grp}]} 0
        if [ "$?" -gt "0" -a "${groups_auto[${grp}]}" == "FALSE" ] ; then
          echo "3##########################"
          echo "group $grp + subgroups should be less than quota of group $grp : ${usage} <= ${groups[${grp}]}"
          condor_userprio -all
          echo "##########################"
          let "ret++"
        fi
      fi
# condor_status
      local usage=0
      local attr=0
      for sub in 1 2; do
         attr=$(condor_status -format "%s\n" "AccountingGroup" | grep "group_${grp}${sub}\.[xy]" | wc -l)
         usage=$(echo -e "scale=100\n${usage} + ${attr}" | bc)
      done
      attr=$(condor_status -format "%s\n" "AccountingGroup" | grep "group_${grp}\.[xy]" | wc -l)
      usage=$(echo -e "scale=100\n${usage} + ${attr}" | bc)
      compare ${usage} ${groups[${grp}]} 0
      if [ "$?" -gt "0" -a "${groups_auto[${grp}]}" == "FALSE" ] ; then
        echo "4##########################"
        echo "group $grp + subgroups should be less than quota of group $grp : ${usage} <= ${groups[${grp}]}"
        condor_status -format "%s\n" "AccountingGroup"
        echo "##########################"
        let "ret++"
      fi
    done
  fi
############### ungrouped jobs
# condor_userprio
  local num_running_jobs_without_AccountingGroup=0
  for i in $(get_numbers_of_ungrouped_users); do
    local attr=$(get_attr ${i} "ResourcesUsed")
    if [ -z "${attr}" ]; then
      attr="0"
    fi
    num_running_jobs_without_AccountingGroup=$((${num_running_jobs_without_AccountingGroup} + ${attr}))
  done
  compare ${num_running_jobs_without_AccountingGroup} ${groups[0]} 0
  if [ "$?" -gt "0" ] ; then
    echo "wrong number of jobs without AccountingGroup: ${num_running_jobs_without_AccountingGroup} <= ${groups[0]}"
    let "ret++"
  fi
# condor_status
  local output=$(condor_q -run -l -attributes "GlobalJobId,AccountingGroup" | grep -v "Submitter" | grep -v "^$")
  local num_running_jobs=$(echo -e ${output} | grep "GlobalJobId" | wc -l)
  local num_running_jobs_with_AccountingGroup=$(echo -e ${output} | grep "AccountingGroup" | wc -l)
  num_running_jobs_without_AccountingGroup=$((${num_running_jobs} - ${num_running_jobs_with_AccountingGroup}))
  compare ${num_running_jobs_without_AccountingGroup} ${groups[0]} 0
  if [ "$?" -gt "0" ] ; then
    echo "wrong number of jobs without AccountingGroup: ${num_running_jobs_without_AccountingGroup} <= ${groups[0]}"
    let "ret++"
  fi
  return ${ret}
}

function compare {
# $1 - $2 <= $3
  awk -v n1=$1 -v n2=$2 -v n3=$3 'BEGIN{if (n1 <= n3 + n2) exit 0; else exit 1}'
  return $?
}

function set_configuration {
  if [ "$TYPE" == "static" ]; then
    {
      echo "NUM_CPUS=${NUM_CPUS}"
      echo "GROUP_NAMES = group_1, group_2"
      echo "GROUP_QUOTA_group_1 = ${NUM_GRP1}"
      echo "GROUP_QUOTA_group_2 = ${NUM_GRP2}"
      echo "GROUP_AUTOREGROUP_group_1 = ${AUTO_GRP1}"
      echo "GROUP_AUTOREGROUP_group_2 = ${AUTO_GRP2}"
    } > ${CONF_FILE}
  fi

  if [ "$TYPE" == "dynamic_as_static" -o "$TYPE" == "dynamic" ]; then
    { 
      echo "NUM_CPUS=${NUM_CPUS}"
      echo "GROUP_NAMES = group_1, group_2, group_1.group_11, group_1.group_12, group_2.group_21, group_2.group_22"
      echo "GROUP_QUOTA_DYNAMIC_group_1 = ${NUM_GRP1}"
      echo "GROUP_QUOTA_DYNAMIC_group_2 = ${NUM_GRP2}"
      echo "GROUP_QUOTA_DYNAMIC_group_1.group_11 = ${NUM_GRP11}"
      echo "GROUP_QUOTA_DYNAMIC_group_1.group_12 = ${NUM_GRP12}"
      echo "GROUP_QUOTA_DYNAMIC_group_2.group_21 = ${NUM_GRP21}"
      echo "GROUP_QUOTA_DYNAMIC_group_2.group_22 = ${NUM_GRP22}"
      echo "GROUP_AUTOREGROUP_group_1 = ${AUTO_GRP1}"
      echo "GROUP_AUTOREGROUP_group_2 = ${AUTO_GRP2}"
      echo "GROUP_AUTOREGROUP_group_1.group_11 = ${AUTO_GRP11}"
      echo "GROUP_AUTOREGROUP_group_1.group_12 = ${AUTO_GRP12}"
      echo "GROUP_AUTOREGROUP_group_2.group_21 = ${AUTO_GRP21}"
      echo "GROUP_AUTOREGROUP_group_2.group_22 = ${AUTO_GRP22}"
    } > ${CONF_FILE}
  fi
 
  if [ "$TYPE" == "dynamic" ]; then 
    groups[1]=$(echo -e "scale=100\n${NUM_GRP1} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
    groups[2]=$(echo -e "scale=100\n${NUM_GRP2} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
    groups[11]=$(echo -e "scale=100\n${NUM_GRP1} * ${NUM_GRP11} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
    groups[12]=$(echo -e "scale=100\n${NUM_GRP1} * ${NUM_GRP12} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
    groups[21]=$(echo -e "scale=100\n${NUM_GRP2} * ${NUM_GRP21} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
    groups[22]=$(echo -e "scale=100\n${NUM_GRP2} * ${NUM_GRP22} * ${NUM_CPUS} + 1" | bc | sed -e "s/\..*//")
  else 
    groups[1]=${NUM_GRP1}
    groups[2]=${NUM_GRP2}
    groups[11]=${NUM_GRP11}
    groups[12]=${NUM_GRP12}
    groups[21]=${NUM_GRP21}
    groups[22]=${NUM_GRP22}
  fi
  #number of slots - all groups = number of slots free for non-grouped jobs
  groups[0]=$((${NUM_CPUS} - ${groups[1]} - ${groups[2]}))
  if [ "${groups[0]}" -lt 1 ]; then
    groups[0]=0
  else
    groups[0]=${NUM_CPUS}
  fi
# jobs without group have same behaviour as groups with autoregroup=true
  groups_auto[0]="TRUE"
  groups_auto[1]=${AUTO_GRP1}
  groups_auto[2]=${AUTO_GRP2}
  groups_auto[11]=${AUTO_GRP11}
  groups_auto[12]=${AUTO_GRP12}
  groups_auto[21]=${AUTO_GRP21}
  groups_auto[22]=${AUTO_GRP22}
}


function run_test {
  {
    if [ "$SUBMIT" == "in_one_queue" ]; then
      echo "cmd = /bin/sleep"
      echo "universe = vanilla"
      echo "should_transfer_files = YES"
      echo "when_to_transfer_output = ON_EXIT"
      echo "arguments = 30"
      echo "queue 100"
    elif [ "$SUBMIT" == "queues" ]; then
      for((i=0;i<100;i++)); do
        echo "cmd = /bin/sleep"
        echo "universe = vanilla"
        echo "should_transfer_files = YES"
        echo "when_to_transfer_output = ON_EXIT"
        echo "arguments = 30"
        echo "queue"
        echo ""
      done
    elif [ "$SUBMIT" == "mixed" ]; then
      echo "cmd = /bin/sleep"
      echo "universe = vanilla"
      echo "should_transfer_files = YES"
      echo "when_to_transfer_output = ON_EXIT"
      echo "arguments = 30"
      echo "queue"
    fi
  } > ${TEST_FILE}

  #BZ613931
  #for user in $(condor_userprio -l | grep Name | awk -F\" '{print $2}'); do condor_userprio -delete $user; done
  condor_rm -all
  service condor stop
  rm -fr /var/lib/condor/spool/*
  rm -fr /var/log/condor/*
  service condor start
  sleep 120
  
  adduser xxx
  adduser xxx1
  adduser xxx2
  adduser yyy
  adduser yyy1
  adduser yyy2
  adduser zzz
  chmod a+rwx /root
  chmod a+rw /root/${TEST_FILE}
  
  if [ "$SUBMIT" == "in_one_queue" -o "$SUBMIT" == "queue" ]; then
    su yyy -c "condor_submit -a '+AccountingGroup = \"group_1.xxx\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su xxx1 -c "condor_submit -a '+AccountingGroup = \"group_1.group_11.xxx1\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su xxx2 -c "condor_submit -a '+AccountingGroup = \"group_1.group_12.xxx2\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    #BZ613931
    #su xxx -c "condor_submit -a '+AccountingGroup = \"group_1\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su yyy -c "condor_submit -a '+AccountingGroup = \"group_2.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su yyy1 -c "condor_submit -a '+AccountingGroup = \"group_2.group_21.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su yyy2 -c "condor_submit -a '+AccountingGroup = \"group_2.group_22.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
    su zzz -c "condor_submit /root/${TEST_FILE}" 1>/dev/null 2>&1
  elif [ "$SUBMIT" == "mixed" ]; then
    for((i=0;i<100;i++)); do
      su yyy -c "condor_submit -a '+AccountingGroup = \"group_1.xxx\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su xxx1 -c "condor_submit -a '+AccountingGroup = \"group_1.group_11.xxx1\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su xxx2 -c "condor_submit -a '+AccountingGroup = \"group_1.group_12.xxx2\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      #BZ613931
      #su xxx -c "condor_submit -a '+AccountingGroup = \"group_1\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su yyy -c "condor_submit -a '+AccountingGroup = \"group_2.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su yyy1 -c "condor_submit -a '+AccountingGroup = \"group_2.group_21.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su yyy2 -c "condor_submit -a '+AccountingGroup = \"group_2.group_22.yyy\"' /root/${TEST_FILE}" 1>/dev/null 2>&1
      su zzz -c "condor_submit /root/${TEST_FILE}" 1>/dev/null 2>&1
    done
  fi
    
  result=0
  jobsq=$(condor_q -run -format '%s\n' 'GlobalJobId' | wc -l)
  if [ -z "${jobsq}" ]; then
    jobsq="0"
  fi
  while [ "${jobsq}" -gt "0" ]; do
    test_resources
    ret="$?"
    if [ "${ret}" -gt "0" ]; then
      result=$((${result} + ${ret}))
    fi
    jobsq=$(condor_q -run -format '%s\n' 'GlobalJobId' | wc -l)
    if [ -z "${jobsq}" ]; then
      jobsq="0"
    fi
  done
  return ${result}
}

#######################################################################################################
#TYPE="static"
#for AUTO_GRP1 in "TRUE" "FALSE"; do
#  for AUTO_GRP2 in "TRUE" "FALSE"; do
#    for submit_i in "mixed" "in_one_queue" "queues"; do
#      SUBMIT=${submit_i}
#
#      # 1st testcase
#      NUM_CPUS=100
#      NUM_GRP1=50
#      NUM_GRP2=25
#      set_configuration
#      run_test
#      ret=${?}
#      if [ "$ret" -gt "0" ]; then
#        ERR=$(($ERR + 1))
#        cat ${TEST_FILE}
#        cat ${CONF_FILE}
#      fi
#      echo "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2}, AUTO_GRP1=${AUTO_GRP1}, AUTO_GRP2=${AUTO_GRP2}: ${ret}"
#      echo "################################################################################"
#      
#      # 2nd testcase
#      NUM_CPUS=75
#      NUM_GRP1=50
#      NUM_GRP2=25
#      set_configuration
#      run_test
#      ret=${?}
#      if [ "$ret" -gt "0" ]; then
#        ERR=$(($ERR + 1))
#        cat ${TEST_FILE}
#        cat ${CONF_FILE}
#      fi
#      echo "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2}, AUTO_GRP1=${AUTO_GRP1}, AUTO_GRP2=${AUTO_GRP2}: ${ret}"
#      echo "################################################################################"
#      
#      # 3rd testcase
#      NUM_CPUS=12
#      NUM_GRP1=50
#      NUM_GRP2=25
#      set_configuration
#      run_test
#      ret=${?}
#      if [ "$ret" -gt "0" ]; then
#        ERR=$(($ERR + 1))
#        cat ${TEST_FILE}
#        cat ${CONF_FILE}
#      fi
#      echo "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2}, AUTO_GRP1=${AUTO_GRP1}, AUTO_GRP2=${AUTO_GRP2}: ${ret}"
#      echo "################################################################################"
#    done
#  done
#done

######################################################################################################
#TYPE="dynamic_as_static"
#for AUTO_GRP1 in "TRUE"; do # parent group has always true autoregroup
#  for AUTO_GRP2 in "TRUE"; do # parent group has always true autoregroup
#    for AUTO_GRP11 in "FALSE" "TRUE"; do
#      for AUTO_GRP12 in "FALSE" "TRUE"; do
#        for AUTO_GRP21 in "FALSE" "TRUE"; do
#          for AUTO_GRP22 in "FALSE" "TRUE"; do
#            for submit_i in  "mixed" "in_one_queue" "queues"; do
#              SUBMIT=${submit_i}
#
#              # 1st testcase
#              NUM_CPUS=100
#              NUM_GRP1=50
#              NUM_GRP2=25
#              NUM_GRP11=25
#              NUM_GRP12=25
#              NUM_GRP21=5
#              NUM_GRP22=10
#              set_configuration
#              run_test
#              ret=${?}
#              if [ "$ret" -gt "0" ]; then
#                ERR=$(($ERR + 1))
#                cat ${TEST_FILE}
#                cat ${CONF_FILE}
#              fi
#              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
#              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
#              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
#              echo "################################################################################"
#  
#              # 2nd testcase
#              NUM_CPUS=75
#              NUM_GRP1=50
#              NUM_GRP2=25
#              set_configuration
#              run_test
#              ret=${?}
#              if [ "$ret" -gt "0" ]; then
#                ERR=$(($ERR + 1))
#                cat ${TEST_FILE}
#                cat ${CONF_FILE}
#              fi
#              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
#              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
#              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
#              echo "################################################################################"
#              
#              # 3rd testcase
#              NUM_CPUS=12
#              NUM_GRP1=50
#              NUM_GRP2=25
#              set_configuration
#              run_test
#              ret=${?}
#              if [ "$ret" -gt "0" ]; then
#                ERR=$(($ERR + 1))
#                cat ${TEST_FILE}
#                cat ${CONF_FILE}
#              fi
#              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
#              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
#              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
#              echo "################################################################################"
#            done
#          done
#        done
#      done
#    done
#  done
#done
#
#####################################################################################################
TYPE="dynamic"
for AUTO_GRP1 in "TRUE"; do # parent group has always true autoregroup
  for AUTO_GRP2 in "TRUE"; do # parent group has always true autoregroup
    for AUTO_GRP11 in "FALSE" "TRUE"; do
      for AUTO_GRP12 in "FALSE" "TRUE"; do
        for AUTO_GRP21 in "FALSE" "TRUE"; do
          for AUTO_GRP22 in "FALSE" "TRUE"; do
            for submit_i in  "mixed" "in_one_queue" "queues"; do
              SUBMIT=${submit_i}

              # 1st testcase
              NUM_CPUS=100
              NUM_GRP1="0.5"
              NUM_GRP2="0.25"
              NUM_GRP11="0.5"
              NUM_GRP12="0.5"
              NUM_GRP21="0.5"
              NUM_GRP22="0.1"
              set_configuration
              run_test
              ret=${?}
              if [ "$ret" -gt "0" ]; then
                ERR=$(($ERR + 1))
                cat ${TEST_FILE}
                cat ${CONF_FILE}
              fi
              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
              echo "################################################################################"
  
              # 2nd testcase
              NUM_CPUS=75
              NUM_GRP1="0.5"
              NUM_GRP2="0.25"
              NUM_GRP11="0.5"
              NUM_GRP12="0.5"
              NUM_GRP21="0.5"
              NUM_GRP22="0.1"
              set_configuration
              run_test
              ret=${?}
              if [ "$ret" -gt "0" ]; then
                ERR=$(($ERR + 1))
                cat ${TEST_FILE}
                cat ${CONF_FILE}
              fi
              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
              echo "################################################################################"
              
              # 3rd testcase
              NUM_CPUS=12
              NUM_GRP1="0.5"
              NUM_GRP2="0.25"
              NUM_GRP11="0.5"
              NUM_GRP12="0.5"
              NUM_GRP21="0.5"
              NUM_GRP22="0.1"
              set_configuration
              run_test
              ret=${?}
              if [ "$ret" -gt "0" ]; then
                ERR=$(($ERR + 1))
                cat ${TEST_FILE}
                cat ${CONF_FILE}
              fi
              echo -e "Result for NUM_CPUS=${NUM_CPUS}, NUM_GRP1=${NUM_GRP1}, NUM_GRP2=${NUM_GRP2},\n\
              NUM_GRP11=${NUM_GRP11}, NUM_GRP12=${NUM_GRP12}, NUM_GRP21=${NUM_GRP21} NUM_GRP22=${NUM_GRP22},\n\
              AUTO_GRP11=${AUTO_GRP11}, AUTO_GRP12=${AUTO_GRP12}, AUTO_GRP21=${AUTO_GRP21}, AUTO_GRP22=${AUTO_GRP22}: ${ret}"
              echo "################################################################################"
            done
          done
        done
      done
    done
  done
done


echo "NUMBER OF ERRORS: ${ERR}"
